﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace multiCFP2
{
    public class Wrapper
    {
        private IntPtr Instance;
        const string DLLPath = @"CFP2API.dll";

        #region DLL Imports
        [DllImport(DLLPath, EntryPoint = "mlcfp2_enumerateDevices", CallingConvention = CallingConvention.StdCall)]
        private static extern int apiEnumerateDevices(UInt16[] Instances, byte[] serialNum);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_createInstance", CallingConvention = CallingConvention.StdCall)]
        private static extern IntPtr apiCreateCFP2Wrapper(UInt16 instNum);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_destroyInstance", CallingConvention = CallingConvention.StdCall)]
        private static extern void apiDestroyCFP2Wrapper(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_connect", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiConnect(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_disconnect", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiDisconnect(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_p3v3CurrentMonitor", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiP3V3CurrentMonitor(IntPtr Instance, ref UInt16 Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_modABS", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiMOD_ABS(IntPtr Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_rxLOS", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiRX_LOS(IntPtr Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_glbALRM", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiGLB_ALRM(IntPtr Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_prgALRM1", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiPRG_ALRM1(IntPtr Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_prgALRM2", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiPRG_ALRM2(IntPtr Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_prgALRM3", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiPRG_ALRM3(IntPtr Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_txDIS", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiTX_DIS(IntPtr Instance, bool isAsserted);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_modLOPWR", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiMOD_LOPWR(IntPtr Instance, bool isAsserted);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_modRST", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiMOD_RST(IntPtr Instance, bool isAsserted);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_prgCNTL1", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiPRG_CNTL1(IntPtr Instance, bool isAsserted);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_prgCNTL2", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiPRG_CNTL2(IntPtr Instance, bool isAsserted);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_prgCNTL3", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiPRG_CNTL3(IntPtr Instance, bool isAsserted);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getSN", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiGet_SN(IntPtr Instance, byte[] SN);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setSN", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiSet_SN(IntPtr Instance, byte[] SN, int SNsize);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setPRTADR", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiSet_PRTADR(IntPtr Instance, byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_readMDIO", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiReadMDIO(IntPtr Instance, byte DeviceAddress, UInt16 RegisterAddress, ref UInt16 Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_writeMDIO", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiWriteMDIO(IntPtr Instance, byte DeviceAddress, UInt16 RegisterAddress, UInt16 Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_eepromRead", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiReadEEPROM(IntPtr Instance, UInt32 address, ref byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_eepromWrite", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiWriteEEPROM(IntPtr Instance, UInt32 address, byte Value);

        #endregion

        /// <summary>Constructs a CFP2 Wrapper.</summary>
        /// <param name="instNum">USB instance number.</param>
        public Wrapper(UInt16 instNum)
        {
            Instance = apiCreateCFP2Wrapper(instNum);
        }

        /// <summary>Enumerates attached devices.</summary>
        /// <returns>Dictionary of attached instances and serial numbers.</returns>
        public static Dictionary<UInt16, string> EnumerateDevices()
        {
            Dictionary<UInt16, string> Devices = new Dictionary<UInt16, string>();
            UInt16[] instances = new UInt16[32];
            string[] SN = new string[32];
            byte[] SNchars = new byte[256]; // every 8 bytes represent an instance's serial number
            int instCount = 0;
            instCount = apiEnumerateDevices(instances, SNchars);
            for (int i = 0; i < instCount; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    if (SNchars[j + i * 8] != '\0')
                    {
                        SN[i] += (char)SNchars[j + i * 8];
                    }
                    else break;
                }
            }
            for (int i = 0; i < instCount; i++)
            {
                Devices.Add(instances[i], SN[i]);
            }
            return Devices;
        }

        /// <summary>Connects to an instance.</summary>
        public bool Connect()
        {
            return apiConnect(Instance);
        }

        /// <summary>Disconnects from an instance.</summary>
        public bool Disconnect()
        {
            return apiDisconnect(Instance);
        }

        /// <summary>Gets Device SN (Needs a FW update).</summary>
        /// <param name="serialNum">returned serial number value.</param>
        /// <returns>true or false.</returns>
        public bool Get_SN(ref string serialNum)
        {
            bool result = false;
            byte[] SNbytes = new byte[8];
            result = apiGet_SN(Instance, SNbytes);
            for (int i = 0; i < 8; i++)
            {
                serialNum += (char)SNbytes[i];
            }
            serialNum = serialNum.TrimEnd('\0');
            return result;
        }

        /// <summary>Sets Device SN (Needs a FW update).</summary>
        /// <param name="serialNum">serial number value.</param>
        /// <returns>true or false.</returns>
        public bool Set_SN(string serialNum)
        {
            byte[] SNbytes;
            int SNsize = serialNum.Length;
            SNbytes = Encoding.ASCII.GetBytes(serialNum);
            return apiSet_SN(Instance, SNbytes, SNsize);
        }

        /// <summary>Reads from EEPROM (Needs a FW update).</summary>
        /// <param name="address">EEPROM Address.</param>
        /// <param name="Value">returned EEPROM Value.</param>
        /// <returns>true or false.</returns>
        public bool ReadEEPROM(UInt32 address, ref byte Value)
        {
            return apiReadEEPROM(Instance, address, ref Value);
        }

        /// <summary>Writes to EEPROM (Needs a FW update).</summary>
        /// <param name="address">EEPROM Address.</param>
        /// <param name="Value">EEPROM Value.</param>
        /// <returns>true or false.</returns>
        public bool WriteEEPROM(UInt32 address, byte Value)
        {
            return apiWriteEEPROM(Instance, address, Value);
        }

        /// <summary>Current Monitor.</summary>
        /// <param name="Value">read current Value in mA.</param>
        /// <returns>true or false.</returns>
        public bool P3V3_Current_Monitor(ref UInt16 Value)
        {
            return apiP3V3CurrentMonitor(Instance, ref Value);
        }

        /// <summary>MODABS pin status.</summary>
        /// <param name="status">true = Module Absent, false = Module Present.</param>
        /// <returns>true or false.</returns>
        public bool MOD_ABS(ref bool status)
        {
            return apiMOD_ABS(Instance, ref status);
        }

        /// <summary>TXDIS pin assertion.</summary>
        /// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>
        /// <returns>true or false.</returns>
        public bool TX_DIS(bool isAsserted)
        {
            return apiTX_DIS(Instance, isAsserted);
        }

        /// <summary>PRGCNTL1 pin assertion.</summary>
        /// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>
        /// <returns>true or false.</returns>
        public bool PRG_CNTL1(bool isAsserted)
        {
            return apiPRG_CNTL1(Instance, isAsserted);
        }

        /// <summary>PRGCNTL2 pin assertion.</summary>
        /// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>
        /// <returns>true or false.</returns>
        public bool PRG_CNTL2(bool isAsserted)
        {
            return apiPRG_CNTL2(Instance, isAsserted);
        }

        /// <summary>PRGCNTL3 pin assertion.</summary>
        /// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>
        /// <returns>true or false.</returns>
        public bool PRG_CNTL3(bool isAsserted)
        {
            return apiPRG_CNTL3(Instance, isAsserted);
        }

        /// <summary>MODRST pin assertion.</summary>
        /// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>
        /// <returns>true or false.</returns>
        public bool MOD_RST(bool isAsserted)
        {
            return apiMOD_RST(Instance, isAsserted);
        }

        /// <summary>GLBALRM pin status.</summary>
        /// <param name="status">true = pin high, false = pin low.</param>
        ///  <returns>true or false.</returns>
        public bool GLB_ALRM(ref bool status)
        {
            return apiGLB_ALRM(Instance, ref status);
        }

        /// <summary>MODLOPWR pin assertion.</summary>
        /// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>
        /// <returns>true or false.</returns>
        public bool MOD_LOPWR(bool isAsserted)
        {
            return apiMOD_LOPWR(Instance, isAsserted);
        }

        /// <summary>Sets MDIO Physical Port Address.</summary>
        /// <param name="Value">Port Value.</param>
        /// <returns>true or false.</returns>
        public bool Set_Add(byte Value)
        {
            return apiSet_PRTADR(Instance, Value);
        }

        /// <summary>Read MDIO.</summary>
        /// <param name="deviceAddress">Device Address.</param>>
        /// <param name="register">Register Address to read from.</param>>
        /// <param name="data">returned register value.</param>>
        public bool Read_MDIO(byte deviceAddress, UInt16 register, ref UInt16 data)
        {
            return apiReadMDIO(Instance, deviceAddress, register, ref data);
        }

        /// <summary>Write MDIO.</summary>
        /// <param name="deviceAddress">Device Address.</param>>
        /// <param name="register">Register address to write to.</param>>
        /// <param name="data">Value to be written.</param>>
        public bool Write_MDIO(byte deviceAddress, UInt16 register, UInt16 data)
        {
            return apiWriteMDIO(Instance, deviceAddress, register, data);
        }

    }
}

